@@ -29,6 +29,7 @@ type ListJSONItem struct {
 	OrigID        string            `json:",omitempty"`
 	Tier          string            `json:",omitempty"`
 	IsBucket      bool              `json:",omitempty"`
+	Metadata      fs.Metadata       `json:",omitempty"`
 }
 
 // Timestamp a time in the provided format
@@ -80,6 +81,7 @@ type ListJSONOpt struct {
 	ShowHash      bool     `json:"showHash"`
 	DirsOnly      bool     `json:"dirsOnly"`
 	FilesOnly     bool     `json:"filesOnly"`
+	Metadata      bool     `json:"metadata"`
 	HashTypes     []string `json:"hashTypes"` // hash types to show if ShowHash is set, e.g. "MD5", "SHA-1"
 }
 
@@ -122,7 +124,7 @@ func newListJSON(ctx context.Context, fsrc fs.Fs, remote string, opt *ListJSONOp
 			return nil, fmt.Errorf("ListJSON failed to load config for crypt remote: %w", err)
 		}
 		if fsInfo.Name != "crypt" {
-			return nil, errors.New("The remote needs to be of type \"crypt\"")
+			return nil, errors.New("the remote needs to be of type \"crypt\"")
 		}
 		lj.cipher, err = crypt.NewCipher(config)
 		if err != nil {
@@ -222,6 +224,14 @@ func (lj *listJSON) entry(ctx context.Context, entry fs.DirEntry) (*ListJSONItem
 				item.Tier = do.GetTier()
 			}
 		}
+		if lj.opt.Metadata {
+			metadata, err := fs.GetMetadata(ctx, x)
+			if err != nil {
+				fs.Errorf(x, "Failed to read metadata: %v", err)
+			} else if metadata != nil {
+				item.Metadata = metadata
+			}
+		}
 	default:
 		fs.Errorf(nil, "Unknown type %T in listing in ListJSON", entry)
 	}
