@@ -39,19 +39,19 @@ type File struct {
 	inode uint64 // inode number - read only
 	size  int64  // size of file - read and written with atomic int64 - must be 64 bit aligned
 
+	muRW sync.Mutex // synchronize RWFileHandle.openPending(), RWFileHandle.close() and File.Remove
+
 	mu               sync.RWMutex                    // protects the following
 	d                *Dir                            // parent directory
 	dPath            string                          // path of parent directory. NB dir rename means all Files are flushed
 	o                fs.Object                       // NB o may be nil if file is being written
 	leaf             string                          // leaf name of the object
 	writers          []Handle                        // writers for this file
-	nwriters         int32                           // len(writers) which is read/updated with atomic
 	pendingModTime   time.Time                       // will be applied once o becomes available, i.e. after file was written
 	pendingRenameFun func(ctx context.Context) error // will be run/renamed after all writers close
-	appendMode       bool                            // file was opened with O_APPEND
 	sys              atomic.Value                    // user defined info to be attached here
-
-	muRW sync.Mutex // synchronize RWFileHandle.openPending(), RWFileHandle.close() and File.Remove
+	nwriters         int32                           // len(writers) which is read/updated with atomic
+	appendMode       bool                            // file was opened with O_APPEND
 }
 
 // newFile creates a new File
@@ -645,15 +645,15 @@ func (f *File) Fs() fs.Fs {
 
 // Open a file according to the flags provided
 //
-//   O_RDONLY open the file read-only.
-//   O_WRONLY open the file write-only.
-//   O_RDWR   open the file read-write.
+//	O_RDONLY open the file read-only.
+//	O_WRONLY open the file write-only.
+//	O_RDWR   open the file read-write.
 //
-//   O_APPEND append data to the file when writing.
-//   O_CREATE create a new file if none exists.
-//   O_EXCL   used with O_CREATE, file must not exist
-//   O_SYNC   open for synchronous I/O.
-//   O_TRUNC  if possible, truncate file when opened
+//	O_APPEND append data to the file when writing.
+//	O_CREATE create a new file if none exists.
+//	O_EXCL   used with O_CREATE, file must not exist
+//	O_SYNC   open for synchronous I/O.
+//	O_TRUNC  if possible, truncate file when opened
 //
 // We ignore O_SYNC and O_EXCL
 func (f *File) Open(flags int) (fd Handle, err error) {
@@ -698,6 +698,11 @@ func (f *File) Open(flags int) (fd Handle, err error) {
 		write = true
 	}
 
+	// If create is set then set write to force openRW
+	if flags&os.O_CREATE != 0 {
+		write = true
+	}
+
 	// Open the correct sort of handle
 	f.mu.RLock()
 	d := f.d
